﻿using System;
using System.Data.Entity;
using System.Linq;

namespace DM
{
    public class DMbase
    {
        protected DbContext db;
        public DMbase(DbContext db)
        {
            this.db = db;
        }

        /// <summary>
        /// select one
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="entity">这里的entity并无实际作用，只是用于编译器推敲类型</param>
        /// <param name="predicate">λ表达式</param>
        /// <returns>返回第一条匹配的记录，若无记录返回null</returns>
        public virtual T FindOne<T>(T entity, Func<T, bool> expression) 
            where T : class, new()
        {
            return this.db.Set<T>().FirstOrDefault(expression);
        }
        
        /// <summary>
        /// select all
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="entity">这里的entity并无实际作用，只是用于编译器推敲类型</param>
        public virtual IQueryable<T> FindAll<T>(T entity) 
            where T : class, new()
        {
            return this.db.Set<T>();
        }

        public virtual void Insert<T>(params T[] entities)
            where T : class, new()
        {
            if (entities != null && entities.Length > 0)
            {
                var set = this.db.Set<T>();
                foreach (var item in entities)
                {
                    set.Add(item);
                }
            }
        }

        public virtual void Delete<T>(params T[] entities)
            where T : class, new()
        {
            if (entities != null && entities.Length > 0)
            {
                var set = this.db.Set<T>();
                foreach (var item in entities)
                {
                    set.Remove(item);
                }
            }
        }

        /// <summary>
        /// 提交事务
        /// </summary>
        public void Commit()
        {
            this.db.SaveChanges();
        }
    }
}
