﻿using DM;
using Factory;
using System;
using System.Linq;
using System.Collections.Generic;
using System.Reflection;

namespace Test
{
    class Program
    {
        static void Main(string[] args)
        {
            var a = ModelFactory.GetTeacher();

            Type aType = a.GetType();
            var aasd = GetEligibleProperties(aType);

            foreach (var item in aasd)
            {
                Console.Write("{0}\n", item.ValueExpression);
            }

            Console.ReadLine();
        }

        private static ModelProperty GetType(PropertyInfo prop)
        {
            ModelProperty a = new ModelProperty();
            Type underlyingType = Nullable.GetUnderlyingType(prop.PropertyType) ?? prop.PropertyType;
            if (prop.GetGetMethod() != null && prop.GetIndexParameters().Length == 0 && IsBindableType(underlyingType))
            {
                a = new ModelProperty
                 {
                     Name = prop.Name,
                     ValueExpression = "Model." + prop.Name,
                     UnderlyingType = underlyingType,
                     IsPrimaryKey = true,
                     IsReadOnly = prop.GetSetMethod() == null
                 };
            }
            return a;
        }

        static List<ModelProperty> GetEligibleProperties(Type type)
        {
            List<ModelProperty> results = new List<ModelProperty>();

            foreach (PropertyInfo prop in type.GetProperties(BindingFlags.Public | BindingFlags.Instance))
            {
                Type underlyingType = Nullable.GetUnderlyingType(prop.PropertyType) ?? prop.PropertyType;
                if (prop.GetGetMethod() != null && prop.GetIndexParameters().Length == 0 && IsBindableType(underlyingType))
                {
                    results.Add(GetType(prop));
                }
                else
                {
                    List<ModelProperty> propertyList = GetEligibleProperties(prop.PropertyType);
                    foreach (var item in propertyList)
                    {
                        results.Add(item);
                    }
                }
            }
            return results;
        }

        static bool IsBindableType(Type type)
        {
            return type.IsPrimitive || bindableNonPrimitiveTypes.Contains(type);
        }

        static Type[] bindableNonPrimitiveTypes = new[] {
    typeof(string),
    typeof(decimal),
    typeof(Guid),
    typeof(DateTime),
    typeof(DateTimeOffset),
    typeof(TimeSpan),
};

        class ModelProperty
        {
            public string Name { get; set; }
            public string ValueExpression { get; set; }
            public Type UnderlyingType { get; set; }
            public bool IsPrimaryKey { get; set; }
            public bool IsReadOnly { get; set; }
        }

        private static void Insert()
        {
            var context = ContextFactory.GetContext();
            var a = ModelFactory.GetTeacher();
            a.ID = Guid.NewGuid();
            a.Pwd = "111";
            a.UserName = "Jinn";
            a.FirstName = "J";
            a.LastName = "W";

            var b = ModelFactory.GetContact();
            b.ID = Guid.NewGuid();
            b.TeacherID = a.ID;
            b.Phone = "asd";
            b.Email = "a@a.c";
            b.CreateTime = DateTime.Now;

            DMbase dm = new DMbase(context);
            dm.Insert(a);
            dm.Insert(b);
            dm.Commit();
        }
    }
}